import datetime

from pydantic import BaseModel

from ..models.uuid import Uuid


class CustomerBalance(BaseModel):
    """A balance for a user.

    This holds information about the financial balance for the user."""

    created_at: datetime.datetime

    id: Uuid

    monthly_credits_remaining: float

    pre_pay_cash_remaining: float

    pre_pay_credits_remaining: float

    total_due: float

    updated_at: datetime.datetime

    user_id: Uuid



from typing import Dict, Optional

from pydantic import BaseModel

from ..models.currency import Currency


class InvoiceLineItem(BaseModel):
    """An invoice line item."""

    amount: Optional[float] = None

    currency: Optional[Currency] = None

    description: Optional[str] = None

    id: Optional[str] = None

    invoice_item: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None



import datetime
from typing import Dict, List, Optional

from pydantic import BaseModel

from ..models.currency import Currency
from ..models.discount import Discount
from ..models.invoice_line_item import InvoiceLineItem
from ..models.invoice_status import InvoiceStatus


class Invoice(BaseModel):
    """An invoice."""

    amount_due: Optional[float] = None

    amount_paid: Optional[float] = None

    amount_remaining: Optional[float] = None

    attempt_count: Optional[int] = None

    attempted: Optional[bool] = None

    created_at: datetime.datetime

    currency: Optional[Currency] = None

    customer_email: Optional[str] = None

    customer_id: Optional[str] = None

    default_payment_method: Optional[str] = None

    description: Optional[str] = None

    discounts: Optional[List[Discount]] = None

    id: Optional[str] = None

    lines: Optional[List[InvoiceLineItem]] = None

    metadata: Optional[Dict[str, str]] = None

    number: Optional[str] = None

    paid: Optional[bool] = None

    pdf: Optional[str] = None

    receipt_number: Optional[str] = None

    statement_descriptor: Optional[str] = None

    status: Optional[InvoiceStatus] = None

    subtotal: Optional[float] = None

    tax: Optional[float] = None

    total: Optional[float] = None

    url: Optional[str] = None



from typing import List

from pydantic import BaseModel

from ..models.export_file import ExportFile


class Export(BaseModel):
    """The response from the `Export` endpoint."""

    files: List[ExportFile]



from typing import Literal, Union

from pydantic import BaseModel, Field, RootModel
from typing_extensions import Annotated

from ..models.center_of_mass import CenterOfMass
from ..models.curve_get_control_points import CurveGetControlPoints
from ..models.curve_get_end_points import CurveGetEndPoints
from ..models.curve_get_type import CurveGetType
from ..models.density import Density
from ..models.entity_get_all_child_uuids import EntityGetAllChildUuids
from ..models.entity_get_child_uuid import EntityGetChildUuid
from ..models.entity_get_distance import EntityGetDistance
from ..models.entity_get_num_children import EntityGetNumChildren
from ..models.entity_get_parent_id import EntityGetParentId
from ..models.entity_linear_pattern import EntityLinearPattern
from ..models.export import Export
from ..models.get_entity_type import GetEntityType
from ..models.get_sketch_mode_plane import GetSketchModePlane
from ..models.highlight_set_entity import HighlightSetEntity
from ..models.import_files import ImportFiles
from ..models.mass import Mass
from ..models.mouse_click import MouseClick
from ..models.path_get_curve_uuids_for_vertices import PathGetCurveUuidsForVertices
from ..models.path_get_info import PathGetInfo
from ..models.path_get_vertex_uuids import PathGetVertexUuids
from ..models.plane_intersect_and_project import PlaneIntersectAndProject
from ..models.select_get import SelectGet
from ..models.select_with_point import SelectWithPoint
from ..models.solid3d_get_all_edge_faces import Solid3dGetAllEdgeFaces
from ..models.solid3d_get_all_opposite_edges import Solid3dGetAllOppositeEdges
from ..models.solid3d_get_next_adjacent_edge import Solid3dGetNextAdjacentEdge
from ..models.solid3d_get_opposite_edge import Solid3dGetOppositeEdge
from ..models.solid3d_get_prev_adjacent_edge import Solid3dGetPrevAdjacentEdge
from ..models.surface_area import SurfaceArea
from ..models.take_snapshot import TakeSnapshot
from ..models.volume import Volume


class empty(BaseModel):
    """An empty response, used for any command that does not explicitly have a response defined here."""

    type: Literal["empty"] = "empty"


class export(BaseModel):
    """The response from the ` ` command."""

    data: Export

    type: Literal["export"] = "export"


class select_with_point(BaseModel):
    """The response from the ` ` command."""

    data: SelectWithPoint

    type: Literal["select_with_point"] = "select_with_point"


class highlight_set_entity(BaseModel):
    """The response from the ` ` command."""

    data: HighlightSetEntity

    type: Literal["highlight_set_entity"] = "highlight_set_entity"


class entity_get_child_uuid(BaseModel):
    """The response from the ` ` command."""

    data: EntityGetChildUuid

    type: Literal["entity_get_child_uuid"] = "entity_get_child_uuid"


class entity_get_num_children(BaseModel):
    """The response from the ` ` command."""

    data: EntityGetNumChildren

    type: Literal["entity_get_num_children"] = "entity_get_num_children"


class entity_get_parent_id(BaseModel):
    """The response from the ` ` command."""

    data: EntityGetParentId

    type: Literal["entity_get_parent_id"] = "entity_get_parent_id"


class entity_get_all_child_uuids(BaseModel):
    """The response from the ` ` command."""

    data: EntityGetAllChildUuids

    type: Literal["entity_get_all_child_uuids"] = "entity_get_all_child_uuids"


class select_get(BaseModel):
    """The response from the ` ` command."""

    data: SelectGet

    type: Literal["select_get"] = "select_get"


class get_entity_type(BaseModel):
    """The response from the ` ` command."""

    data: GetEntityType

    type: Literal["get_entity_type"] = "get_entity_type"


class entity_get_distance(BaseModel):
    """The response from the ` ` command."""

    data: EntityGetDistance

    type: Literal["entity_get_distance"] = "entity_get_distance"


class entity_linear_pattern(BaseModel):
    """The response from the ` ` command."""

    data: EntityLinearPattern

    type: Literal["entity_linear_pattern"] = "entity_linear_pattern"


class solid3d_get_all_edge_faces(BaseModel):
    """The response from the ` ` command."""

    data: Solid3dGetAllEdgeFaces

    type: Literal["solid3d_get_all_edge_faces"] = "solid3d_get_all_edge_faces"


class solid3d_get_all_opposite_edges(BaseModel):
    """The response from the ` ` command."""

    data: Solid3dGetAllOppositeEdges

    type: Literal["solid3d_get_all_opposite_edges"] = "solid3d_get_all_opposite_edges"


class solid3d_get_opposite_edge(BaseModel):
    """The response from the ` ` command."""

    data: Solid3dGetOppositeEdge

    type: Literal["solid3d_get_opposite_edge"] = "solid3d_get_opposite_edge"


class solid3d_get_prev_adjacent_edge(BaseModel):
    """The response from the ` ` command."""

    data: Solid3dGetPrevAdjacentEdge

    type: Literal["solid3d_get_prev_adjacent_edge"] = "solid3d_get_prev_adjacent_edge"


class solid3d_get_next_adjacent_edge(BaseModel):
    """The response from the ` ` command."""

    data: Solid3dGetNextAdjacentEdge

    type: Literal["solid3d_get_next_adjacent_edge"] = "solid3d_get_next_adjacent_edge"


class mouse_click(BaseModel):
    """The response from the ` ` command."""

    data: MouseClick

    type: Literal["mouse_click"] = "mouse_click"


class curve_get_type(BaseModel):
    """The response from the ` ` command."""

    data: CurveGetType

    type: Literal["curve_get_type"] = "curve_get_type"


class curve_get_control_points(BaseModel):
    """The response from the ` ` command."""

    data: CurveGetControlPoints

    type: Literal["curve_get_control_points"] = "curve_get_control_points"


class take_snapshot(BaseModel):
    """The response from the ` ` command."""

    data: TakeSnapshot

    type: Literal["take_snapshot"] = "take_snapshot"


class path_get_info(BaseModel):
    """The response from the ` ` command."""

    data: PathGetInfo

    type: Literal["path_get_info"] = "path_get_info"


class path_get_curve_uuids_for_vertices(BaseModel):
    """The response from the ` ` command."""

    data: PathGetCurveUuidsForVertices

    type: Literal[
        "path_get_curve_uuids_for_vertices"
    ] = "path_get_curve_uuids_for_vertices"


class path_get_vertex_uuids(BaseModel):
    """The response from the ` ` command."""

    data: PathGetVertexUuids

    type: Literal["path_get_vertex_uuids"] = "path_get_vertex_uuids"


class plane_intersect_and_project(BaseModel):
    """The response from the ` ` command."""

    data: PlaneIntersectAndProject

    type: Literal["plane_intersect_and_project"] = "plane_intersect_and_project"


class curve_get_end_points(BaseModel):
    """The response from the ` ` command."""

    data: CurveGetEndPoints

    type: Literal["curve_get_end_points"] = "curve_get_end_points"


class import_files(BaseModel):
    """The response from the ` ` command."""

    data: ImportFiles

    type: Literal["import_files"] = "import_files"


class mass(BaseModel):
    """The response from the ` ` command."""

    data: Mass

    type: Literal["mass"] = "mass"


class volume(BaseModel):
    """The response from the ` ` command."""

    data: Volume

    type: Literal["volume"] = "volume"


class density(BaseModel):
    """The response from the ` ` command."""

    data: Density

    type: Literal["density"] = "density"


class surface_area(BaseModel):
    """The response from the ` ` command."""

    data: SurfaceArea

    type: Literal["surface_area"] = "surface_area"


class center_of_mass(BaseModel):
    """The response from the ` ` command."""

    data: CenterOfMass

    type: Literal["center_of_mass"] = "center_of_mass"


class get_sketch_mode_plane(BaseModel):
    """The response from the ` ` command."""

    data: GetSketchModePlane

    type: Literal["get_sketch_mode_plane"] = "get_sketch_mode_plane"


OkModelingCmdResponse = RootModel[
    Annotated[
        Union[
            empty,
            export,
            select_with_point,
            highlight_set_entity,
            entity_get_child_uuid,
            entity_get_num_children,
            entity_get_parent_id,
            entity_get_all_child_uuids,
            select_get,
            get_entity_type,
            entity_get_distance,
            entity_linear_pattern,
            solid3d_get_all_edge_faces,
            solid3d_get_all_opposite_edges,
            solid3d_get_opposite_edge,
            solid3d_get_prev_adjacent_edge,
            solid3d_get_next_adjacent_edge,
            mouse_click,
            curve_get_type,
            curve_get_control_points,
            take_snapshot,
            path_get_info,
            path_get_curve_uuids_for_vertices,
            path_get_vertex_uuids,
            plane_intersect_and_project,
            curve_get_end_points,
            import_files,
            mass,
            volume,
            density,
            surface_area,
            center_of_mass,
            get_sketch_mode_plane,
        ],
        Field(discriminator="type"),
    ]
]



import datetime
from typing import Optional

from pydantic import BaseModel

from ..models.uuid import Uuid


class ExtendedUser(BaseModel):
    """Extended user information.

    This is mostly used for internal purposes. It returns a mapping of the user's information, including that of our third party services we use for users: MailChimp, Stripe, and Front
    """

    company: Optional[str] = None

    created_at: datetime.datetime

    discord: Optional[str] = None

    email: Optional[str] = None

    email_verified: Optional[datetime.datetime] = None

    first_name: Optional[str] = None

    front_id: Optional[str] = None

    github: Optional[str] = None

    id: Uuid

    image: str

    last_name: Optional[str] = None

    mailchimp_id: Optional[str] = None

    name: Optional[str] = None

    phone: Optional[str] = None

    stripe_id: Optional[str] = None

    updated_at: datetime.datetime



import datetime
from typing import Dict, Optional

from pydantic import BaseModel

from ..models.cluster import Cluster
from ..models.gateway import Gateway
from ..models.jetstream import Jetstream
from ..models.leaf_node import LeafNode


class Connection(BaseModel):
    """Metadata about a pub-sub connection.

    This is mostly used for internal purposes and debugging."""

    auth_timeout: Optional[int] = None

    cluster: Optional[Cluster] = None

    config_load_time: datetime.datetime

    connections: Optional[int] = None

    cores: Optional[int] = None

    cpu: Optional[float] = None

    gateway: Optional[Gateway] = None

    git_commit: Optional[str] = None

    go: Optional[str] = None

    gomaxprocs: Optional[int] = None

    host: str

    http_base_path: Optional[str] = None

    http_host: Optional[str] = None

    http_port: Optional[int] = None

    http_req_stats: Dict[str, int]

    https_port: Optional[int] = None

    in_bytes: Optional[int] = None

    in_msgs: Optional[int] = None

    jetstream: Optional[Jetstream] = None

    leaf: Optional[LeafNode] = None

    leafnodes: Optional[int] = None

    max_connections: Optional[int] = None

    max_control_line: Optional[int] = None

    max_payload: Optional[int] = None

    max_pending: Optional[int] = None

    mem: Optional[int] = None

    now: datetime.datetime

    out_bytes: Optional[int] = None

    out_msgs: Optional[int] = None

    ping_interval: Optional[int] = None

    ping_max: Optional[int] = None

    port: Optional[int] = None

    proto: Optional[int] = None

    remotes: Optional[int] = None

    routes: Optional[int] = None

    server_id: Optional[str] = None

    server_name: Optional[str] = None

    slow_consumers: Optional[int] = None

    start: datetime.datetime

    subscriptions: Optional[int] = None

    system_account: Optional[str] = None

    tls_timeout: Optional[int] = None

    total_connections: Optional[int] = None

    uptime: Optional[str] = None

    version: Optional[str] = None

    write_deadline: Optional[int] = None



from typing import List, Optional

from pydantic import BaseModel

from ..models.api_call_with_price import ApiCallWithPrice


class ApiCallWithPriceResultsPage(BaseModel):
    """A single page of results"""

    items: List[ApiCallWithPrice]

    next_page: Optional[str] = None



from typing import List, Optional

from pydantic import BaseModel

from ..models.text_to_cad import TextToCad


class TextToCadResultsPage(BaseModel):
    """A single page of results"""

    items: List[TextToCad]

    next_page: Optional[str] = None



from typing import Optional

from pydantic import BaseModel

from ..models.ai_plugin_api_type import AiPluginApiType


class AiPluginApi(BaseModel):
    """AI plugin api information."""

    is_user_authenticated: Optional[bool] = None

    type: Optional[AiPluginApiType] = None

    url: str



from typing import List

from pydantic import BaseModel



class PathGetCurveUuidsForVertices(BaseModel):
    """The response from the `PathGetCurveUuidsForVertices` command."""

    curve_ids: List[str]



from typing import Optional

from pydantic import BaseModel



class Coupon(BaseModel):
    """The resource representing a Coupon."""

    amount_off: Optional[float] = None

    deleted: Optional[bool] = None

    id: Optional[str] = None

    percent_off: Optional[float] = None



from typing import Optional

from pydantic import BaseModel

from ..models.jetstream_config import JetstreamConfig
from ..models.jetstream_stats import JetstreamStats
from ..models.meta_cluster_info import MetaClusterInfo


class Jetstream(BaseModel):
    """Jetstream information."""

    config: Optional[JetstreamConfig] = None

    meta: Optional[MetaClusterInfo] = None

    stats: Optional[JetstreamStats] = None



from typing import Optional

from pydantic import BaseModel

from ..models.country_code import CountryCode
from ..models.uuid import Uuid


class NewAddress(BaseModel):
    """The struct that is used to create a new record. This is automatically generated and has all the same fields as the main struct only it is missing the `id`."""

    city: Optional[str] = None

    country: CountryCode

    state: Optional[str] = None

    street1: Optional[str] = None

    street2: Optional[str] = None

    user_id: Uuid

    zip: Optional[str] = None




from pydantic import BaseModel



class DeviceAuthVerifyParams(BaseModel):
    """The request parameters to verify the `user_code` for the OAuth 2.0 Device Authorization Grant."""

    user_code: str



from typing import Optional

from pydantic import BaseModel

from ..models.ai_plugin_api import AiPluginApi
from ..models.ai_plugin_auth import AiPluginAuth


class AiPluginManifest(BaseModel):
    """AI plugin manifest.

    This is used for OpenAI's ChatGPT plugins. You can read more about them [here](https://platform.openai.com/docs/plugins/getting-started/plugin-manifest).
    """

    api: AiPluginApi

    auth: AiPluginAuth

    contact_email: Optional[str] = None

    description_for_human: Optional[str] = None

    description_for_model: Optional[str] = None

    legal_info_url: str

    logo_url: str

    name_for_human: Optional[str] = None

    name_for_model: Optional[str] = None

    schema_version: Optional[str] = None



from typing import Optional

from pydantic import BaseModel



class JetstreamConfig(BaseModel):
    """Jetstream configuration."""

    domain: Optional[str] = None

    max_memory: Optional[int] = None

    max_storage: Optional[int] = None

    store_dir: Optional[str] = None



from typing import Optional

from pydantic import BaseModel



class MetaClusterInfo(BaseModel):
    """Jetstream statistics."""

    cluster_size: Optional[int] = None

    leader: Optional[str] = None

    name: Optional[str] = None



from typing import Any, Dict, Optional

import httpx

from ...client import Client
from ...models.ai_feedback import AiFeedback
from ...models.error import Error
from ...types import Response


def _get_kwargs(
    id: str,
    feedback: AiFeedback,
    *,
    client: Client,
) -> Dict[str, Any]:
    url = "{}/user/text-to-cad/{id}".format(
        client.base_url,
        id=id,
    )  # noqa: E501

    if feedback is not None:
        if "?" in url:
            url = url + "&feedback=" + str(feedback)
        else:
            url = url + "?feedback=" + str(feedback)

    headers: Dict[str, Any] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    return {
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
    }


def _parse_response(*, response: httpx.Response) -> Optional[Error]:
    return None
    if response.status_code == 400:
        response_4XX = Error(**response.json())
        return response_4XX
    if response.status_code == 500:
        response_5XX = Error(**response.json())
        return response_5XX
    return Error(**response.json())


def _build_response(*, response: httpx.Response) -> Response[Optional[Error]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    id: str,
    feedback: AiFeedback,
    *,
    client: Client,
) -> Response[Optional[Error]]:
    kwargs = _get_kwargs(
        id=id,
        feedback=feedback,
        client=client,
    )

    response = httpx.post(
        verify=client.verify_ssl,
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    id: str,
    feedback: AiFeedback,
    *,
    client: Client,
) -> Optional[Error]:
    """This endpoint requires authentication by any Zoo user. The user must be the owner of the text-to-CAD model, in order to give feedback."""  # noqa: E501

    return sync_detailed(
        id=id,
        feedback=feedback,
        client=client,
    ).parsed


async def asyncio_detailed(
    id: str,
    feedback: AiFeedback,
    *,
    client: Client,
) -> Response[Optional[Error]]:
    kwargs = _get_kwargs(
        id=id,
        feedback=feedback,
        client=client,
    )

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        response = await _client.post(**kwargs)

    return _build_response(response=response)


async def asyncio(
    id: str,
    feedback: AiFeedback,
    *,
    client: Client,
) -> Optional[Error]:
    """This endpoint requires authentication by any Zoo user. The user must be the owner of the text-to-CAD model, in order to give feedback."""  # noqa: E501

    return (
        await asyncio_detailed(
            id=id,
            feedback=feedback,
            client=client,
        )
    ).parsed



from typing import Optional

from pydantic import BaseModel



class Onboarding(BaseModel):
    """Onboarding details"""

    first_call_from_modeling_app_date: Optional[str] = None

    first_call_from_text_to_cad_date: Optional[str] = None

    first_call_from_their_machine_date: Optional[str] = None

    first_litterbox_execute_date: Optional[str] = None

    first_token_date: Optional[str] = None



from typing import List

from pydantic import BaseModel

from ..models.path_segment_info import PathSegmentInfo


class PathGetInfo(BaseModel):
    """The response from the `PathGetInfo` command."""

    segments: List[PathSegmentInfo]



from typing import Optional

from pydantic import BaseModel



class Solid3dGetNextAdjacentEdge(BaseModel):
    """The response from the `Solid3dGetNextAdjacentEdge` command."""

    edge: Optional[str] = None



import datetime

from pydantic import BaseModel

from ..models.uuid import Uuid


class Session(BaseModel):
    """An authentication session.

    For our UIs, these are automatically created by Next.js."""

    created_at: datetime.datetime

    expires: datetime.datetime

    id: Uuid

    session_token: Uuid

    updated_at: datetime.datetime

    user_id: Uuid




from pydantic import BaseModel

from ..models.rtc_sdp_type import RtcSdpType


class RtcSessionDescription(BaseModel):
    """SessionDescription is used to expose local and remote session descriptions."""

    sdp: str

    type: RtcSdpType



from typing import List

from pydantic import BaseModel

from ..models.point3d import Point3d


class CurveGetControlPoints(BaseModel):
    """The response from the `CurveGetControlPoints` command."""

    control_points: List[Point3d]



from typing import Optional

from pydantic import BaseModel

from ..models.ai_plugin_auth_type import AiPluginAuthType
from ..models.ai_plugin_http_auth_type import AiPluginHttpAuthType


class AiPluginAuth(BaseModel):
    """AI plugin auth information."""

    authorization_type: Optional[AiPluginHttpAuthType] = None

    type: Optional[AiPluginAuthType] = None



from typing import List, Optional

from pydantic import BaseModel

from ..models.ai_prompt import AiPrompt


class AiPromptResultsPage(BaseModel):
    """A single page of results"""

    items: List[AiPrompt]

    next_page: Optional[str] = None



import datetime
from typing import Dict, Optional

from pydantic import BaseModel

from ..models.currency import Currency
from ..models.new_address import NewAddress


class Customer(BaseModel):
    """The resource representing a payment "Customer"."""

    address: Optional[NewAddress] = None

    balance: Optional[float] = None

    created_at: datetime.datetime

    currency: Optional[Currency] = None

    delinquent: Optional[bool] = None

    email: Optional[str] = None

    id: Optional[str] = None

    metadata: Optional[Dict[str, str]] = None

    name: Optional[str] = None

    phone: Optional[str] = None
